package turgen.plugin.plturbo;

import turgen.plugin.*;
import turgen.generator.SignalGenerator;
import turgen.config.Configuration;
import turgen.*;
import java.io.*;
import javax.swing.*;
import turgen.config.BooleanConfigEntry;
import turgen.config.ConfigClass;
import turgen.config.IntegerConfigEntry;
import turgen.dtb.DOS2Binary;
import turgen.playlist.AbstractPlayListItem;
import turgen.tapeimage.DataChunk;
import turgen.tapeimage.TapeImageChunk;
import turgen.tapeimage.TurboChunk;


public class PlKSOTurbo2000Plugin implements Plugin {
    
    
    private String lastError;
    private PlTurboPluginPanel panel;
    private InOutProvider ioProvider;
    private int[] casChunkName={116,114,107,115};

    private ConfigClass cfcPlKSOTurbo;
    private IntegerConfigEntry cfgPilotLenA;
    private IntegerConfigEntry cfgPilotLenBlock;
    private BooleanConfigEntry cfgInvertPolarity;
    
    /** Nova instance PlKSOTurbo2000Plugin */
    public PlKSOTurbo2000Plugin() {
    }
    
    /** Jake rychlosti jsou podporovany*/
    
    
    /** Identifikacni jmeno zasuvneho modulu*/
    public String getName() {
        return "KSO Turbo 2000";
    }
    
   
    
    
    /*************************************************************/
    
    /**Tvorba instrukci pro generator*/
    public int[] getInstructions(AbstractPlayListItem _pl) {
        
        int[] instr=null;
        File tmpf=null;
        
        DataOutputStream dos = null;
        RandomAccessFile raf = null;
        DataInputStream dis = null;
        
        SimplePlaylistItem pl;
        pl=(SimplePlaylistItem)_pl;
        try {
            
            /*Otevreni vsech souboru a docasnych proudu*/
            raf = new RandomAccessFile(pl.inFile,"r");
            int l = (int)raf.length();
            
            if (l>1024*1024) throw new FileTooLargeException(pl.inFile);

            /*Read all data from file and close it*/
            byte[] fData = new byte[l];
            int[] fileData = new int[l];

            raf.readFully(fData);
            raf.close();

            for(int i=0;i<fData.length;i++) {
                fileData[i]=fData[i]<0?fData[i]+256:fData[i];
            }

            /*Check for binary file*/
            int[] initLocations;
            try {
                DOS2Binary dtb = new DOS2Binary(pl.inFile);
                dtb.analyzeFromData(fileData);
                initLocations = dtb.getInitLocations();
            }
            catch (Exception e) {
                initLocations = new int[0];
            }
            
            
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            dos = new DataOutputStream(baos);
            
            /*Nastaveni*/
            Utils.addSetupInstruction(dos, pl.pulses, casChunkName,1-cfgInvertPolarity.getAsInt(),SignalGenerator.FLAG_ORDER_HL);
            
            /*Hlavicka, zavadeci ton, 10 znaku nazev, suma vznikla scitanim*/
            dos.writeInt(SignalGenerator.INSTR_PILOTTONE);
            dos.writeInt(cfgPilotLenA.getAsInt());
            dos.writeInt(SignalGenerator.INSTR_DATA); 
            dos.writeInt(13);
            
            int[] header = new int[13];
            /*Pocatek hlavicky*/
            header[0]=0;
            header[1]=255;
            
            /*Vymezerovat nazev*/
            for(int i=2;i<12;i++) {
                header[i]=32;
            }
            
            int max=10;
            int hl = pl.fname.length();
            if (hl>max) hl=max;
            
            /*Jmeno souboru*/
            for(int i=2;i<12;i++) {
                if ((i-2)<hl) header[i]=(byte)pl.fname.charAt(i-2); else header[i]=32;
            }
            
            /*Kontrolni soucet*/
            int chsum=0;
            for(int i=0;i<12;i++) {
                chsum+=header[i];
                dos.writeInt(header[i]);
            }
            dos.writeInt(chsum % 256);
            dos.writeInt(SignalGenerator.INSTR_NARROW);
            
            /*Pauza 1 sekundu*/
            dos.writeInt(SignalGenerator.INSTR_LONGSILENCE);
            dos.writeInt(1);
            
            /*ZAPIS JEDNOTLIVYCH BLOKU*/
            int[] block = new int[3072+2+1];
            int fptr=0;
            
            int rest=0;

            int startPtr;
            int endPtr;
            
            /*Pokud neni vse zapsano*/
            while (fptr<l) {

                startPtr=fptr;

                /*Zapsat zavadeci ton*/
                dos.writeInt(SignalGenerator.INSTR_PILOTTONE);
                dos.writeInt(cfgPilotLenBlock.getAsInt());
                dos.writeInt(SignalGenerator.INSTR_DATA);
                dos.writeInt(3072+2+1);
                
                /*Kolik zbyva do konce a je-li to vice nez 3072 bytu*/
                rest=l-fptr;
                if (rest>3072) rest=3072;
                
                /*Delka bloku*/
                block[0]=rest % 256;
                block[1]=rest / 256;
                
                
                /*Data bloku*/
                for(int i=0;i<rest;i++) {
                    block[i+2]=fileData[fptr];fptr++;
                }
                
                /*Doplnit nulami*/
                for (int i=rest;i<3072;i++) {
                    block[i+2]=0;
                }
                
                /*Kontrolni soucet*/
                chsum=0;
                for (int i=0;i<3074;i++) {
                    chsum+=block[i];
                    dos.writeInt(block[i]);
                }
                
                dos.writeInt(chsum % 256);

                endPtr=fptr;

                /*Safety pulse*/
                dos.writeInt(SignalGenerator.INSTR_NARROW);

                /*Silence after init or simple block separation*/
                int wait=Utils.calculateInitSilence(startPtr, endPtr-1, initLocations, pl.pauses);
                if (wait==0) {
                    dos.writeInt(SignalGenerator.INSTR_BLOCKSEP);
                }
                else {
                    dos.writeInt(SignalGenerator.INSTR_LONGSILENCE);
                    dos.writeInt(Math.min(240, wait));
                }
                
                
            }
            /*Byl-li posledni blok dlouhy 3072 bytu, tak je potreba zapsat
             *EOF blok
             */
            if (rest==3072) {
                dos.writeInt(SignalGenerator.INSTR_PILOTTONE);
                dos.writeInt(cfgPilotLenBlock.getAsInt());
                dos.writeInt(SignalGenerator.INSTR_DATA);
                dos.writeInt(3072+2+1);
                for(int i=0;i<3072+2+1;i++) {
                    dos.writeInt(0);
                }
                
            }
            /*Safety pulse + safety pause*/
            dos.writeInt(SignalGenerator.INSTR_NARROW);
            dos.writeInt(SignalGenerator.INSTR_BLOCKSEP);
            
            
            dos.close();
            
            

            byte[] bk = baos.toByteArray();
            int cntr = bk.length/4;

            dis = new DataInputStream(new ByteArrayInputStream(bk));
            
            /*Instrukce jsou zapsany, predani generatoru*/
            instr=new int[cntr+1];

            for(int i=0;i<cntr;i++) {
                instr[i]=dis.readInt();
            }
            instr[cntr]=SignalGenerator.INSTR_END;
            
            dis.close();
            return instr;
     
            
        }
        
        catch(Exception e) {
            e.printStackTrace();
            if (e.getMessage()==null) lastError="Unable to produce instructions: "+e.toString();
            else lastError=e.toString();
            return null;
            
        }
        
        finally {
            try {
            raf.close();
            dis.close();
            dos.close();
            }
            catch(Exception e) {
                /*No reactions*/
            }
        }
       
        
    }
    
    public String getLastError() {
        return lastError;
    }
    
    public AbstractPlayListItem createItem() {
        SimplePlaylistItem dpr = panel.getRecord();
        dpr.pluginClass=Turgen.pluginManager.getPluginClassName(this);
        return dpr;
    }
    
    public void resetPanel() {
        panel.clear();
    }
    
    public String validateInput() {
        return panel.verify();
    }
    
    public JPanel getItemPanel() {
        return panel;
    }
    
    public void init() {
        panel = new PlTurboPluginPanel(10);

        /*Initialize configuration class*/
        cfcPlKSOTurbo = new ConfigClass("PlKSOTurbo", "KSO Turbo 2000");

        /*Initialize configuration entries*/
        cfgPilotLenA=new IntegerConfigEntry("pilotLenA","Header block pilot tone length [pulse]",3072,256,8192);
        cfgPilotLenBlock=new IntegerConfigEntry("pilotLenBlock","Data block pilot tone length [pulse]",2048,256,8192);
        cfgInvertPolarity =new BooleanConfigEntry("invertPolarity","Invert polarity of pulses",false);

        /*Add entries to the class*/
        cfcPlKSOTurbo.addEntry(cfgPilotLenA);
        cfcPlKSOTurbo.addEntry(cfgPilotLenBlock);
        cfcPlKSOTurbo.addEntry(cfgInvertPolarity);

        /*Register configuration class*/
        Configuration.registerClass(cfcPlKSOTurbo);
    }
    
    public void fillPanel(AbstractPlayListItem plr) {
        SimplePlaylistItem dpr = (SimplePlaylistItem)plr;
        panel.fill(dpr);
    }


    public void setInOutProvider(InOutProvider prov) {
        ioProvider=prov;
        panel.ioProvider=prov;
    }

     public String[] getSupportedTapeImageChunks() {
        return new String[] {"trks"};
    }

     public void addInstructionsForChunk(DataOutputStream dos, TapeImageChunk chunk) throws Exception {

        /*trXX chunk*/
        if (chunk instanceof TurboChunk) {
            Utils.addSetupInstruction(dos, Turgen.pulseCache.getPulsesForPlugin(this)[0], this.casChunkName,1-cfgInvertPolarity.getAsInt(),SignalGenerator.FLAG_ORDER_HL);
            return;
        }

        /*Only data chunk accepted*/
        if (!(chunk instanceof DataChunk)) return;
        DataChunk dataChunk = (DataChunk)chunk;

        
        

        /*Zavadeci ton a data bloku*/
        dos.writeInt(SignalGenerator.INSTR_PILOTTONE);
        dos.writeInt(dataChunk.getAux());

        /*Data bloku*/
        dos.writeInt(SignalGenerator.INSTR_DATA);
        dos.writeInt(dataChunk.getLength());

        int[] dta = dataChunk.getData();

        for(int i=0;i<dataChunk.getLength();i++) {
            dos.writeInt(dta[i]);
        }

        /*Bezpecnostni pulz*/
        dos.writeInt(SignalGenerator.INSTR_NARROW);

    }

    public BinaryConvertor[] getBinaryConvertors() {
        BinaryConvertor[] bcs = new BinaryConvertor[1];
        bcs[0]=new PlKSOTurbo2000BinaryConvertor(this, "KSO Turbo 2000");

        return bcs;
    }

    
    public String getCountry() {
        return "PL";
    }

    public boolean isUsingAlternateFileChooser() {
        return false;
    }

    
    
}
